VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "Math"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
Attribute VB_Description = "This class exposes a large number of ready-to-use derived mathematical functions.\r\n\r\nCONSTANTS\r\nProvides 15-16 digit constants for Pi, e, and Euler's constant.\r\n\r\nNON-TRIG FUNCTIONS\r\nSeveral of these are trivial implementations to make conversion of Javascript code simpler or to simplify coding otherwise. These include:\r\n+ floor, ceil: return same values as Javascript floor and ceil would.\r\n+ Ackerman, Fibonacci calculation routines.\r\n+ isOdd and isEven functions to determine whether a numbe is odd or even.  These will NOT return anticipated values for a real number which has some error.\r\n+ Comparison functions: min and max compare two values; this differs from Fortran standard, which does a ParamArray-style comparison. The fcmp function provides a float comparison for non-precise values.\r\n+ GreatestCommonFactor, LeastCommonMultiple for a pair of digits.\r\n+ Some useful derived functions such as log10, LogBaseN, and hypot(enuse).\r\n+ Product and Summation functions for a single-dimensioned array of numbers.\r\n\r\nTRIGONOMETRIC FUNCTIONS\r\nTrig functions can generally be identified as follows.  Inverse functions begin with ""A"" for ""arc"".  Hyperbolic functions end with ""H"".\r\nSome core trig functions are provided as Fortran analogs which take arguments in degrees instead of radians; these all end in ""d"".\r\n+ Cartesian and polar/spherical coordinate transformation.\r\n+ Degree/radian conversions for convenience.\r\n+ a ""range2Pi"" boolean function for testing whether one is within the 0 to 2 Pi range of values."
Attribute VB_Ext_KEY = "SavedWithClassBuilder6" ,"Yes"
Attribute VB_Ext_KEY = "Top_Level" ,"Yes"
' Copyright (C) 2003  Alex K. Angelopoulos, aka@mvps.org
'
' This library is free software; you can redistribute it and/or
' modify it under the terms of the GNU Lesser General Public
' License as published by the Free Software Foundation; either
' version 2.1 of the License, or (at your option) any later version.
'
' This library is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
' Lesser General Public License for more details.
'
' You should have received a copy of the GNU Lesser General Public
' License along with this library; if not, write to the
' Free Software Foundation, Inc.
' 59 Temple Place, Suite 330
' Boston, MA  02111-1307  USA
'


Option Explicit

Public Property Get e() As Double
Attribute e.VB_Description = "The natural exponential number, e, to 15 places."
  e = 2.71828182845905 ' e
End Property

Public Property Get eulersConstant() As Double
Attribute eulersConstant.VB_Description = "Euler's constant to 15 places."
  eulersConstant = 0.577215664901533 ' euler constant
End Property

Public Property Get Pi() As Double
Attribute Pi.VB_Description = "Pi to 15 places."
  Pi = 3.14159265358979
End Property

Public Function fcmp(ByVal x As Double, ByVal y As Double, ByVal epsilon As Double) As Boolean
Attribute fcmp.VB_Description = "Tests the numbers for equality within range epsilon.  If difference is less than epsilon, then they are considered equal"
  If Abs(x - y) <= epsilon Then
    fcmp = True
  Else
    fcmp = False
  End If
End Function


Public Function ACos(ByVal x As Double) As Double
Attribute ACos.VB_Description = "Inverse cosine; returns a value in radians."
  ' returns empty value for undefined solutions
  On Error Resume Next
  'Inverse Cosine
  ACos = Atn(-x / Sqr(-x * x + 1)) + 2 * Atn(1)
End Function

Public Function acosd(ByVal value As Double) As Double
Attribute acosd.VB_Description = "Inverse cosine; returns a value in degrees."
  acosd = ACos(value) * 180 / Pi
End Function

Public Function ACosH(ByVal x As Double) As Double
Attribute ACosH.VB_Description = "Inverse hyperbolic cosine; returns a value in radians."
  ' returns empty value for undefined solutions
  On Error Resume Next
  'Inverse Hyperbolic Cosine
  ACosH = Log(x + Sqr(x * x - 1))
End Function

Public Function ACoSec(ByVal x As Double) As Double
Attribute ACoSec.VB_Description = "Inverse cosecant; returns a value in radians."
  ' returns empty value for undefined solutions
  On Error Resume Next
  'Inverse Cosec
  ACoSec = Atn(x / Sqr(x * x - 1)) + (Sgn(x) - 1) * (2 * Atn(1))
End Function

Public Function ACosecH(ByVal x As Double) As Double
Attribute ACosecH.VB_Description = "Inverse hyperbolic cosecant; returns a value in radians."
  ' returns empty value for undefined solutions
  On Error Resume Next
  'Inverse Hyperbolic Cosec
  ACosecH = Log((Sgn(x) * Sqr(x * x + 1) + 1) / x)
End Function

Public Function ACoTan(ByVal x As Double) As Double
Attribute ACoTan.VB_Description = "Inverse cotangent; returns a value in radians."
  ' returns empty value for undefined solutions
  On Error Resume Next
  'Inverse Cotangent
  ACoTan = Atn(x) + 2 * Atn(1)
End Function

Public Function ACotanH(ByVal x As Double) As Double
Attribute ACotanH.VB_Description = "Inverse hyperbolic cotangent; returns a value in radians."
  ' returns empty value for undefined solutions
  On Error Resume Next
  'Inverse Hyperbolic Cotangent
  ACotanH = Log((x + 1) / (x - 1)) / 2
End Function

Public Function ASec(ByVal x As Double) As Double
Attribute ASec.VB_Description = "Inverse secant; returns a value in radians."
  ' returns empty value for undefined solutions
  On Error Resume Next
  'Inverse Secant
  ASec = Atn(x / Sqr(x * x - 1)) + Sgn((x) - 1) * (2 * Atn(1))
End Function

Public Function AsecH(ByVal x As Double) As Double
Attribute AsecH.VB_Description = "Inverse hyperbolic secant; returns a value in radians."
  ' returns empty value for undefined solutions
  On Error Resume Next
  'Inverse Hyperbolic Secant
  AsecH = Log((Sqr(-x * x + 1) + 1) / x)
End Function

Public Function ASin(ByVal x As Double) As Double
Attribute ASin.VB_Description = "Inverse sine; returns a value in radians."
  ' returns empty value for undefined solutions
  On Error Resume Next
  'Inverse Sine
  ASin = Atn(x / Sqr(-x * x + 1))
End Function

Public Function asind(ByVal value As Double) As Double
Attribute asind.VB_Description = "Sine using degrees."
  asind = ASin(value) * 180 / Pi
End Function

Public Function AsinH(ByVal x As Double) As Double
Attribute AsinH.VB_Description = "Inverse hyperbolic sine; returns a value in radians."
  ' returns empty value for undefined solutions
  On Error Resume Next
  AsinH = Log(x + Sqr(x * x + 1))
End Function

Public Function atan2(x, y)
Attribute atan2.VB_Description = "Second-quadrant inverse tangent."
  ' from the Draw Arrows tip
  ' @ http://www.devx.com/upload/free/features/VBPJ/techtips/techtips11.pdf
  ' derived from Jim Deutch's work
  Dim theta As Double
  If x <> 0 Then
    theta = Atn(y / x)
    If x < 0 Then
      theta = theta + Pi
    End If
  Else
    If y < 0 Then
      theta = 3 * Pi / 2 '90
    Else
      theta = Pi / 2 ' 270
    End If
  End If
  atan2 = theta
End Function

Public Function atand(ByVal value As Double) As Double
Attribute atand.VB_Description = "Arctangent; returns degrees."
  atand = Atn(value) * 180 / Pi
End Function

Public Function AtanH(ByVal x As Double) As Double
Attribute AtanH.VB_Description = "Inverse hyperbolic cosecant; returns a value in radians."
  ' returns empty value for undefined solutions
  On Error Resume Next
  'Inverse Hyperbolic Tangent
  AtanH = Log((1 + x) / (1 - x)) / 2
End Function

Public Function cosd(ByVal value As Double) As Double
Attribute cosd.VB_Description = "Cosine using degrees."
  cosd = Cos(value * Pi / 180)
End Function

Public Function CosH(ByVal x As Double) As Double
Attribute CosH.VB_Description = "Hyperbolic cosine."
  ' returns empty value for undefined solutions
  On Error Resume Next
  'Hyperbolic Cosine
  CosH = (Exp(x) + Exp(-x)) / 2
End Function

Public Function Cosec(ByVal x As Double) As Double
Attribute Cosec.VB_Description = "Cosecant."
  ' returns empty value for undefined solutions
  On Error Resume Next
  Cosec = 1 / Sin(x)
End Function

Public Function CosecH(ByVal x As Double) As Double
Attribute CosecH.VB_Description = "Hyperbolic cosecant; returns a value in radians."
  ' returns empty value for undefined solutions
  On Error Resume Next
  'Hyperbolic Cosec
  CosecH = 2 / (Exp(x) - Exp(-x))
End Function

Public Function Cotan(ByVal x As Double) As Double
Attribute Cotan.VB_Description = "Cotangent."
  ' returns empty value for undefined solutions
  On Error Resume Next
  Cotan = 1 / Tan(x)
End Function

Public Function CotanH(ByVal x As Double) As Double
Attribute CotanH.VB_Description = "Hyperbolic cotangent."
  ' returns empty value for undefined solutions
  On Error Resume Next
  'Hyperbolic Cotangent
  CotanH = (Exp(x) + Exp(-x)) / (Exp(x) - Exp(-x))
End Function

Public Function Secant(ByVal x As Double) As Double
Attribute Secant.VB_Description = "Secant."
  ' returns empty value for undefined solutions
  On Error Resume Next
  Secant = 1 / Cos(x)
End Function

Public Function SecH(ByVal x As Double) As Double
Attribute SecH.VB_Description = "Hyperbolic Secant."
  ' returns empty value for undefined solutions
  On Error Resume Next
  'Hyperbolic Secant
  SecH = 2 / (Exp(x) + Exp(-x))
End Function

Public Function sind(ByVal value As Double) As Double
Attribute sind.VB_Description = "Sine function using degrees."
  sind = Sin(value * Pi / 180)
End Function

Public Function SinH(ByVal x As Double) As Double
Attribute SinH.VB_Description = "Inverse hyperbolic sine; returns a value in radians."
  ' returns empty value for undefined solutions
  On Error Resume Next
  'Hyperbolic Sine
  SinH = (Exp(x) - Exp(-x)) / 2
End Function

Public Function tand(ByVal degrees As Double) As Double
Attribute tand.VB_Description = "Tangent using degrees."
  tand = Tan(degrees * Pi / 180)
End Function

Public Function TanH(ByVal radians As Double) As Double
Attribute TanH.VB_Description = "hyperbolic tangent; takes an argument in radians."
  ' returns empty value for undefined solutions
  On Error Resume Next
  'Hyperbolic Tangent
  TanH = (Exp(radians) - Exp(-radians)) / (Exp(radians) + Exp(-radians))
End Function

Public Function range2pi(ByVal x As Double) As Double
Attribute range2pi.VB_Description = "returns an angle x in the range 0 to 2*Pi radians."
  '   returns an angle x in the range 0 to two pi rads
  range2pi = x - 2 * Pi * Int(x / (2 * Pi))
End Function

Public Function LogBaseN(ByVal x As Double, ByVal n As Double) As Double
Attribute LogBaseN.VB_Description = "Calculates logarithm with base n of x."
  ' returns empty value for undefined solutions
  On Error Resume Next
  'Logarithm to base N
  LogBaseN = Log(x) / Log(n)
End Function

Public Function log10(ByVal value As Double) As Double
Attribute log10.VB_Description = "Calculates log base 10 of value."
  log10 = Log(value) / Log(10)
End Function

Public Function Cart3ToSpherical(ByVal x As Double, ByVal y As Double, ByVal z As Double)
Attribute Cart3ToSpherical.VB_Description = "Given an (x, y, z) ordered triple of Cartesian 3-space coordinates, translates them into spherical coordinates and returns a (radius, latitude, longitude) ordered triple."
  ' Returns radius, latitude, longitude values in 3-element array
  ' drawn from Mike Hapgood's reference algorithms
  ' http://sspg1.bnsc.rl.ac.uk/Share/Coordinates/ct_home.htm
  Dim radius As Double, latitude As Double, longitude As Double
  radius = Sqr(x ^ 2 + y ^ 2 + z ^ 2)
  latitude = Atn(z / Sqr(x ^ 2 + y ^ 2))
  If y > 0 Then
    longitude = ACos(x / Sqr(x ^ 2 + y ^ 2))
  Else
    longitude = 4 * Atn(1) - ACos(x / Sqr(x ^ 2 + y ^ 2))
  End If
  Cart3ToSpherical = Array(radius, latitude, longitude)
End Function

Public Function SphericaltoCart3(radius, latitude, longitude)
Attribute SphericaltoCart3.VB_Description = "Given spherical coordinates (radius, latitude, longitude), translates them into Cartesian 3-space coordinates and returns an (x, y, z) ordered triple."
  ' NOTE - latitude and longitude must be in RADIANS
  ' use Pi/180 or the degreesToRadians function if necessary
  ' drawn from Mike Hapgood's reference algorithms
  ' http://sspg1.bnsc.rl.ac.uk/Share/Coordinates/ct_home.htm
  ' Returns x, y, z values in 3-element array
  Dim x As Double, y As Double, z As Double
  x = radius * Cos(latitude) * Cos(longitude)
  y = radius * Cos(latitude) * Sin(longitude)
  z = radius * Sin(latitude)
  SphericaltoCart3 = Array(x, y, z)
End Function

Public Function CartesianToPolar(ByVal x As Double, ByVal y As Double)
Attribute CartesianToPolar.VB_Description = "Given x-y Cartesian coordinates, translates them into polar coordinates and returns a (radius, theta [radians]) value."
  ' Returns r, theta values in 2-element array
  Dim radius As Double, theta As Double
  radius = Sqr(x * x + y * y)
  ' only try to find a value if radius is not 0
  ' if it is 0, we will return an empty value
  ' as a logical analog to "undefined"
  If radius > 0 Then
    theta = atan2(x, y)
  End If
  CartesianToPolar = Array(radius, theta)
End Function

Public Function radiansToDegrees(radians)
Attribute radiansToDegrees.VB_Description = "Converts a value in radians to degrees."
  radiansToDegrees = 180 * radians / Pi
End Function

Public Function degreesToRadians(degrees)
Attribute degreesToRadians.VB_Description = "Converts a value in degrees to radians."
  degreesToRadians = Pi * degrees / 180
End Function

Public Function floor(ByVal value As Double) As Long
Attribute floor.VB_Description = "Finds the greatest integer less than or equal to value.  Provides a result identical to the Javascript floor function; this is simply a wrapper around Visual Basic's Int() function."
  floor = Int(value)
End Function

Public Function ceil(ByVal value As Double) As Long
Attribute ceil.VB_Description = "Finds the smallest integer greater than or equal to value.  Provides a result identical to the Javascript ceil function; this is simply a wrapper around Visual Basic's Int() function with (1) added."
  ceil = Int(value) + 1
End Function

Public Function Fibonacci(ByVal value As Long) As Long
Attribute Fibonacci.VB_Description = "Calculates Fibonacci function for a value.  This is a non-recursive Fibonacci algorithm."
  ' non-recursive Fibonacci calculation.
  ' based on the VBScript implementation maintained by Jerry Yoakum
  ' at csc.smsu.edu
  Dim n1, n2, tmp
  n1 = 1
  n2 = 1
  tmp = 0
  Dim i
  For i = 2 To value
    tmp = n1 + n2
    n1 = n2
    n2 = tmp
  Next
  Fibonacci = n1
End Function

Public Function Ackerman(ByVal m As Long, n As Long) As Long
Attribute Ackerman.VB_Description = "returns Ackerman's function for m and n.  This is a recursive Ackerman function."
  If m = 0 Then
    Ackerman = n + 1
  Else
    If n = 0 Then
      Ackerman = Ackerman(m - 1, 1)
    Else
      Ackerman = Ackerman(m - 1, Ackerman(m, n - 1))
    End If
  End If
End Function

Public Function LeastCommonMultiple(ByVal int1 As Long, ByVal int2 As Long) As Long
Attribute LeastCommonMultiple.VB_Description = "Determines least common multiple of long integers int1 and int2."
  'Finds the least common multiple of two integers
  LeastCommonMultiple = Abs(int1 * int2 / GreatestCommonFactor(int1, int2))
End Function

Public Function GreatestCommonFactor(ByVal int1 As Long, ByVal int2 As Long) As Long
Attribute GreatestCommonFactor.VB_Description = "Determines greatest common factor of long integers int1 and int2."
  'Finds the greatest common factor of two integers
  Dim r As Long
  Dim lTmp As Long
  Dim done As Boolean

  If int1 < 0 Then int1 = Abs(int1)
  If int2 < 0 Then int2 = Abs(int2)
  Select Case True
    Case int1 = int2
      GreatestCommonFactor = int1: Exit Function
    Case int1 < int2
      lTmp = int2: int2 = int1: int1 = lTmp
  End Select

  Do While done = False
    r = int1 Mod int2
    If r = 0 Then Exit Do
    int1 = int2
    int2 = r
  Loop
  GreatestCommonFactor = int2
End Function

Public Function Summation(ParamArray Values() As Variant) As Double
Attribute Summation.VB_Description = "returns sum of Values()."
  Dim i As Long
  For i = LBound(Values) To UBound(Values)
    Summation = Summation + Values(i)
  Next
End Function

Public Function Product(ParamArray Values() As Variant) As Double
Attribute Product.VB_Description = "returns mathematical product of Values()."
  Dim i As Long
  Product = 1
  For i = LBound(Values) To UBound(Values)
    Product = Product + Values(i)
  Next
End Function

Public Function max(ByVal a As Variant, ByVal B As Variant) As Variant
Attribute max.VB_Description = "returns greater of a and b."
  max = IIf(a > B, a, B)
End Function


Public Function min(ByVal a As Variant, ByVal B As Variant) As Variant
Attribute min.VB_Description = "returns lesser of a and b."
  min = IIf(a < B, a, B)
End Function

Public Function hypot(ByVal x As Double, ByVal y As Double) As Double
Attribute hypot.VB_Description = "returns hypotenuse corresponding to x and y as right-angle sides."
  ' calculates hypotenuse
  hypot = Sqr(x ^ 2 + y ^ 2)
End Function


Public Function isOdd(ByVal value As Long) As Boolean
Attribute isOdd.VB_Description = "Tests value and returns true if it is an odd integer, false if not."
  isOdd = IIf((value + 1) \ 2 = (value + 1) / 2, True, False)
End Function

Public Function isEven(ByVal value As Long) As Boolean
Attribute isEven.VB_Description = "Tests value and returns true if it is an even integer, false if not."
  isEven = IIf(value \ 2 = value / 2, True, False)
End Function
